@@ -12,34 +12,38 @@
 
 #include "kleopatra_debug.h"
 
-#include "crypto/taskcollection.h"
 #include "crypto/decryptverifytask.h"
-#include "crypto/gui/resultpage.h"
 #include "crypto/gui/resultlistwidget.h"
+#include "crypto/gui/resultpage.h"
+#include "crypto/taskcollection.h"
 #include "utils/path-helper.h"
 
 #include <Libkleo/FileNameRequester>
 
-#include <QWindow>
-#include <QVBoxLayout>
-#include <QProgressBar>
 #include <QLabel>
+#include <QProgressBar>
 #include <QPushButton>
+#include <QVBoxLayout>
+#include <QWindow>
 
 #include <vector>
 
+#include <KConfigGroup>
+#include <KLocalizedContext>
 #include <KLocalizedString>
 #include <KMessageBox>
-#include <KConfigGroup>
 #include <KSharedConfig>
-#include <KWindowConfig>
+#include <MimeTreeParserWidgets/MessageViewerDialog>
 
 using namespace Kleo;
 using namespace Kleo::Crypto;
 using namespace Kleo::Crypto::Gui;
+using namespace MimeTreeParser::Widgets;
 
 DecryptVerifyFilesDialog::DecryptVerifyFilesDialog(const std::shared_ptr<TaskCollection> &coll, QWidget *parent)
-    : QDialog(parent), m_tasks(coll), m_buttonBox(new QDialogButtonBox)
+    : QDialog(parent)
+    , m_tasks(coll)
+    , m_buttonBox(new QDialogButtonBox)
 {
     readConfig();
     auto vLay = new QVBoxLayout(this);
@@ -47,7 +51,7 @@ DecryptVerifyFilesDialog::DecryptVerifyFilesDialog(const std::shared_ptr<TaskCol
     auto outputLayout = new QHBoxLayout;
 
     m_outputLocationFNR = new FileNameRequester;
-    auto outLabel = new QLabel(i18n("&Output folder:"));
+    auto outLabel = new QLabel(i18nc("@label:textbox", "&Output folder:"));
     outLabel->setBuddy(m_outputLocationFNR);
     outputLayout->addWidget(outLabel);
     outputLayout->addWidget(m_outputLocationFNR);
@@ -60,6 +64,7 @@ DecryptVerifyFilesDialog::DecryptVerifyFilesDialog(const std::shared_ptr<TaskCol
     m_progressBar = new QProgressBar;
     vLay->addWidget(m_progressBar);
     m_resultList = new ResultListWidget;
+    connect(m_resultList, &ResultListWidget::showButtonClicked, this, &DecryptVerifyFilesDialog::showContent);
     vLay->addWidget(m_resultList);
 
     m_tasks = coll;
@@ -75,7 +80,7 @@ DecryptVerifyFilesDialog::DecryptVerifyFilesDialog(const std::shared_ptr<TaskCol
     layout()->addWidget(m_buttonBox);
 
     bool hasOutputs = false;
-    for (const auto &t: coll->tasks()) {
+    for (const auto &t : coll->tasks()) {
         if (!qobject_cast<VerifyDetachedTask *>(t.get())) {
             hasOutputs = true;
             break;
@@ -111,7 +116,7 @@ void DecryptVerifyFilesDialog::allDone()
     Q_ASSERT(m_tasks);
     m_progressBar->setRange(0, 100);
     m_progressBar->setValue(100);
-    for (const auto &i: m_progressLabelByTag.keys()) {
+    for (const auto &i : m_progressLabelByTag.keys()) {
         if (!i.isEmpty()) {
             m_progressLabelByTag.value(i)->setText(i18n("%1: All operations completed.", i));
         } else {
@@ -187,11 +192,11 @@ void DecryptVerifyFilesDialog::btnClicked(QAbstractButton *btn)
     }
 }
 
-void DecryptVerifyFilesDialog::checkAccept() {
+void DecryptVerifyFilesDialog::checkAccept()
+{
     const auto outLoc = outputLocation();
     if (outLoc.isEmpty()) {
-        KMessageBox::information(this, i18n("Please select an output folder."),
-                                 i18nc("@title:window", "No Output Folder"));
+        KMessageBox::information(this, i18n("Please select an output folder."), i18nc("@title:window", "No Output Folder"));
         return;
     }
     const QFileInfo fi(outLoc);
@@ -200,17 +205,24 @@ void DecryptVerifyFilesDialog::checkAccept() {
         qCDebug(KLEOPATRA_LOG) << "Output dir does not exist. Trying to create.";
         const QDir dir(outLoc);
         if (!dir.mkdir(outLoc)) {
-            KMessageBox::information(this, xi18nc("@info", "<para>Failed to create output folder <filename>%1</filename>.</para><para>Please select a different output folder.</para>", outLoc),
-                                     i18nc("@title:window", "Unusable Output Folder"));
+            KMessageBox::information(
+                this,
+                xi18nc("@info",
+                       "<para>Failed to create output folder <filename>%1</filename>.</para><para>Please select a different output folder.</para>",
+                       outLoc),
+                i18nc("@title:window", "Unusable Output Folder"));
         } else {
             accept();
         }
     } else if (!fi.isDir()) {
-        KMessageBox::information(this, i18n("Please select a different output folder."),
-                                 i18nc("@title:window", "Invalid Output Folder"));
+        KMessageBox::information(this, i18n("Please select a different output folder."), i18nc("@title:window", "Invalid Output Folder"));
     } else if (!Kleo::isWritable(fi)) {
-        KMessageBox::information(this, xi18nc("@info", "<para>Cannot write in the output folder <filename>%1</filename>.</para><para>Please select a different output folder.</para>", outLoc),
-                                 i18nc("@title:window", "Unusable Output Folder"));
+        KMessageBox::information(
+            this,
+            xi18nc("@info",
+                   "<para>Cannot write in the output folder <filename>%1</filename>.</para><para>Please select a different output folder.</para>",
+                   outLoc),
+            i18nc("@title:window", "Unusable Output Folder"));
     } else {
         accept();
     }
@@ -218,24 +230,26 @@ void DecryptVerifyFilesDialog::checkAccept() {
 
 void DecryptVerifyFilesDialog::readConfig()
 {
-    winId(); // ensure there's a window created
-
-    // set default window size
-    windowHandle()->resize(640, 480);
-
-    // restore size from config file
-    KConfigGroup cfgGroup(KSharedConfig::openStateConfig(), "DecryptVerifyFilesDialog");
-    KWindowConfig::restoreWindowSize(windowHandle(), cfgGroup);
-
-    // NOTICE: QWindow::setGeometry() does NOT impact the backing QWidget geometry even if the platform
-    // window was created -> QTBUG-40584. We therefore copy the size here.
-    // TODO: remove once this was resolved in QWidget QPA
-    resize(windowHandle()->size());
+    KConfigGroup dialog(KSharedConfig::openStateConfig(), QStringLiteral("DecryptVerifyFilesDialog"));
+    const QSize size = dialog.readEntry("Size", QSize(640, 480));
+    if (size.isValid()) {
+        resize(size);
+    }
 }
 
 void DecryptVerifyFilesDialog::writeConfig()
 {
-    KConfigGroup cfgGroup(KSharedConfig::openStateConfig(), "DecryptVerifyFilesDialog");
-    KWindowConfig::saveWindowSize(windowHandle(), cfgGroup);
-    cfgGroup.sync();
+    KConfigGroup dialog(KSharedConfig::openStateConfig(), QStringLiteral("DecryptVerifyFilesDialog"));
+    dialog.writeEntry("Size", size());
+    dialog.sync();
 }
+
+void DecryptVerifyFilesDialog::showContent(const std::shared_ptr<const Task::Result> &result)
+{
+    if (auto decryptVerifyResult = std::dynamic_pointer_cast<const DecryptVerifyResult>(result)) {
+        MessageViewerDialog dialog(decryptVerifyResult->fileName());
+        dialog.exec();
+    }
+}
+
+#include "moc_decryptverifyfilesdialog.cpp"
