@@ -32,6 +32,7 @@
 #include <Libkleo/Algorithm>
 #include <Libkleo/DefaultKeyFilter>
 #include <Libkleo/Formatting>
+#include <Libkleo/GnuPG>
 #include <Libkleo/KeyCache>
 #include <Libkleo/KeyHelpers>
 #include <Libkleo/KeySelectionCombo>
@@ -115,13 +116,6 @@ public:
     using TreeWidget::TreeWidget;
 
 protected:
-    void focusInEvent(QFocusEvent *event) override
-    {
-        TreeWidget::focusInEvent(event);
-        // queue the invokation, so that it happens after the widget itself got focus
-        QMetaObject::invokeMethod(this, &TreeWidgetInternal::forceAccessibleFocusEventForCurrentItem, Qt::QueuedConnection);
-    }
-
     bool edit(const QModelIndex &index, EditTrigger trigger, QEvent *event) override
     {
         if (event && event->type() == QEvent::KeyPress) {
@@ -133,17 +127,6 @@ protected:
         }
         return TreeWidget::edit(index, trigger, event);
     }
-
-private:
-    void forceAccessibleFocusEventForCurrentItem()
-    {
-        // force Qt to send a focus event for the current item to accessibility
-        // tools; otherwise, the user has no idea which item is selected when the
-        // list gets keyboard input focus
-        const auto current = currentIndex();
-        setCurrentIndex({});
-        setCurrentIndex(current);
-    }
 };
 
 struct UserIDCheckState {
@@ -256,11 +239,16 @@ public:
             auto layout = new QHBoxLayout;
 
             mPublishCB = new QCheckBox{q};
-            mPublishCB->setText(i18n("Publish on keyserver afterwards"));
+            if (keyserver().startsWith(QLatin1StringView("ldap:")) || keyserver().startsWith(QLatin1StringView("ldaps:"))) {
+                mPublishCB->setText(i18nc("@label:checkbox", "Publish in internal directory"));
+            } else {
+                mPublishCB->setText(i18nc("@label:checkbox", "Publish to %1", keyserver()));
+            }
             mPublishCB->setToolTip(xi18nc("@info:tooltip",
-                                          "Check this option, if you want to upload your certifications to a certificate "
+                                          "Check this option if you want to upload your certifications to a certificate "
                                           "directory after successful certification."));
-            mPublishCB->setEnabled(mExportCB->isChecked());
+            mPublishCB->setEnabled(mExportCB->isChecked() && haveKeyserverConfigured());
+            mPublishCB->setVisible(haveKeyserverConfigured());
 
             layout->addSpacing(checkBoxSize(mExportCB).width());
             layout->addWidget(mPublishCB);
@@ -349,7 +337,7 @@ public:
         });
 
         connect(mExportCB, &QCheckBox::toggled, q, [this](bool on) {
-            mPublishCB->setEnabled(on);
+            mPublishCB->setEnabled(on && haveKeyserverConfigured());
         });
 
         connect(mSecKeySelect, &KeySelectionCombo::currentKeyChanged, q, [this](const GpgME::Key &) {
@@ -396,13 +384,13 @@ public:
         switch (mMode) {
         case SingleCertification: {
             mExportCB->setChecked(conf.readEntry("ExportCheckState", false));
-            mPublishCB->setChecked(conf.readEntry("PublishCheckState", false));
+            mPublishCB->setChecked(conf.readEntry("PublishCheckState", false) && haveKeyserverConfigured());
             mAdvancedOptionsExpander->setExpanded(conf.readEntry("AdvancedOptionsExpanded", false));
             break;
         }
         case BulkCertification: {
             mExportCB->setChecked(conf.readEntry("BulkExportCheckState", true));
-            mPublishCB->setChecked(conf.readEntry("BulkPublishCheckState", false));
+            mPublishCB->setChecked(conf.readEntry("BulkPublishCheckState", false) && haveKeyserverConfigured());
             mAdvancedOptionsExpander->setExpanded(conf.readEntry("BulkAdvancedOptionsExpanded", true));
             break;
         }
@@ -642,7 +630,7 @@ public:
             return true;
         }
 
-        const auto allTagsAreLoaded = Kleo::all_of(mKeys, [](const auto &key) {
+        const auto allTagsAreLoaded = std::ranges::all_of(mKeys, [](const auto &key) {
             return (key.protocol() != GpgME::OpenPGP) || (key.keyListMode() & GpgME::SignatureNotations);
         });
         if (allTagsAreLoaded) {
@@ -677,7 +665,7 @@ public:
             updateTrustSignatureDomain();
         } else {
             // check for certificates that cannot be certified
-            const auto haveBadCertificates = Kleo::any_of(mKeys, [](const auto &key) {
+            const auto haveBadCertificates = std::ranges::any_of(mKeys, [](const auto &key) {
                 const auto &uid = key.userID(0);
                 return (key.protocol() != OpenPGP) || uid.isInvalid() || Kleo::isRevokedOrExpired(uid);
             });
@@ -727,7 +715,7 @@ public:
         for (int i = 0, end = userIdListView->topLevelItemCount(); i < end; ++i) {
             const auto uidItem = userIdListView->topLevelItem(i);
             const auto itemUserId = getUserId(uidItem);
-            const bool userIdIsInList = Kleo::any_of(uids, [itemUserId](const auto &uid) {
+            const bool userIdIsInList = std::ranges::any_of(uids, [itemUserId](const auto &uid) {
                 return Kleo::userIDsAreEqual(itemUserId, uid);
             });
             uidItem->setCheckState(0, userIdIsInList ? Qt::Checked : Qt::Unchecked);
