@@ -20,7 +20,6 @@
 #include "view/keylistcontroller.h"
 #include "view/padwidget.h"
 #include "view/searchbar.h"
-#include "view/smartcardwidget.h"
 #include "view/tabwidget.h"
 #include "view/welcomewidget.h"
 
@@ -40,6 +39,7 @@
 
 #include <Libkleo/GnuPG>
 
+#include "dialogs/debugdialog.h"
 #include "dialogs/updatenotification.h"
 
 // needed for GPGME_VERSION_NUMBER
@@ -128,7 +128,6 @@ namespace
 {
 static const std::vector<QString> mainViewActionNames = {
     QStringLiteral("view_certificate_overview"),
-    QStringLiteral("manage_smartcard"),
     QStringLiteral("pad_view"),
 };
 
@@ -168,7 +167,7 @@ private:
 
             searchBar = new SearchBar{q};
             vbox->addWidget(searchBar);
-            tabWidget = new TabWidget{q};
+            tabWidget = new TabWidget{KeyTreeView::Option::NoDefaultContextMenu, q};
             vbox->addWidget(tabWidget);
 
             tabWidget->connectSearchBar(searchBar);
@@ -358,11 +357,6 @@ public:
         }
     }
 
-    void showSmartcardView()
-    {
-        showView(QStringLiteral("manage_smartcard"), ui.scWidget);
-    }
-
     void showPadView()
     {
         if (!ui.padWidget) {
@@ -407,7 +401,7 @@ private:
     void keyListingDone()
     {
         const auto curWidget = ui.stackWidget->currentWidget();
-        if (curWidget == ui.scWidget || curWidget == ui.padWidget) {
+        if (curWidget == ui.padWidget) {
             return;
         }
         showCertificateView();
@@ -419,7 +413,6 @@ private:
     struct UI {
         CertificateView *searchTab = nullptr;
         PadWidget *padWidget = nullptr;
-        SmartCardWidget *scWidget = nullptr;
         WelcomeWidget *welcomeWidget = nullptr;
         QStackedWidget *stackWidget = nullptr;
         explicit UI(MainWindow *q);
@@ -442,9 +435,6 @@ MainWindow::Private::UI::UI(MainWindow *q)
 
     new KeyCacheOverlay(mainWidget, q);
 
-    scWidget = new SmartCardWidget{q};
-    stackWidget->addWidget(scWidget);
-
     welcomeWidget = new WelcomeWidget{q};
     stackWidget->addWidget(welcomeWidget);
 
@@ -459,13 +449,13 @@ MainWindow::Private::Private(MainWindow *qq)
     , firstShow(true)
     , ui(q)
 {
-    KDAB_SET_OBJECT_NAME(controller);
+    Q_SET_OBJECT_NAME(controller);
 
     AbstractKeyListModel *flatModel = AbstractKeyListModel::createFlatKeyListModel(q);
     AbstractKeyListModel *hierarchicalModel = AbstractKeyListModel::createHierarchicalKeyListModel(q);
 
-    KDAB_SET_OBJECT_NAME(flatModel);
-    KDAB_SET_OBJECT_NAME(hierarchicalModel);
+    Q_SET_OBJECT_NAME(flatModel);
+    Q_SET_OBJECT_NAME(hierarchicalModel);
 
 #if GPGME_VERSION_NUMBER >= 0x011800 // 1.24.0
     auto keyExportDragHandler = std::make_shared<KeyExportDragHandler>();
@@ -589,6 +579,19 @@ void MainWindow::Private::setupActions()
             QString(),
         },
 #endif
+        {
+            "tools_debug_view",
+            i18n("Debug View"),
+            QString(),
+            "",
+            q,
+            [this](bool) {
+                auto dialog = new DebugDialog(q);
+                dialog->setAttribute(Qt::WA_DeleteOnClose);
+                dialog->open();
+            },
+            QString(),
+        },
         {
             "tools_restart_backend",
             i18nc("@action:inmenu", "Restart Background Processes"),
@@ -643,8 +646,8 @@ void MainWindow::Private::setupActions()
             i18n("Show smartcard management"),
             "auth-sim-locked",
             q,
-            [this](bool) {
-                showSmartcardView();
+            [](bool) {
+                KleopatraApplication::instance()->openOrRaiseSmartCardWindow();
             },
             QString(),
         },
@@ -704,7 +707,7 @@ void MainWindow::Private::setupActions()
     KStandardAction::keyBindings(q, SLOT(editKeybindings()), coll);
     KStandardAction::preferences(qApp, SLOT(openOrRaiseConfigDialog()), coll);
 
-    focusToClickSearchAction = new QAction(i18n("Set Focus to Quick Search"), q);
+    focusToClickSearchAction = new QAction(i18nc("@action", "Set Focus to Quick Search"), q);
     coll->addAction(QStringLiteral("focus_to_quickseach"), focusToClickSearchAction);
     coll->setDefaultShortcut(focusToClickSearchAction, QKeySequence(Qt::ALT | Qt::Key_Q));
     connect(focusToClickSearchAction, SIGNAL(triggered(bool)), q, SLOT(slotFocusQuickSearch()));
@@ -731,7 +734,7 @@ void MainWindow::Private::setupActions()
      * the focus on official communications. */
     const auto quickguide =
         new DocAction(QIcon::fromTheme(QStringLiteral("help-contextual")),
-                      i18n("&Quickguide"),
+                      i18n("&Quick Guide Encrypt and Sign"),
                       i18nc("Only available in German and English. Leave to English for other languages.", "encrypt_and_sign_gnupgvsd_en.pdf"),
                       QStringLiteral("../share/doc/gnupg-vsd"),
                       QUrl(),
@@ -740,7 +743,7 @@ void MainWindow::Private::setupActions()
 
     const auto symguide =
         new DocAction(QIcon::fromTheme(QStringLiteral("help-contextual")),
-                      i18n("&Password-based encryption"),
+                      i18n("&Password-based Encryption"),
                       i18nc("Only available in German and English. Leave to English for other languages.", "symmetric_encryption_gnupgvsd_en.pdf"),
                       QStringLiteral("../share/doc/gnupg-vsd"),
                       QUrl(),
@@ -748,7 +751,7 @@ void MainWindow::Private::setupActions()
     coll->addAction(QStringLiteral("help_doc_symenc"), symguide);
 
     const auto groups = new DocAction(QIcon::fromTheme(QStringLiteral("help-contextual")),
-                                      i18n("&Group configuration"),
+                                      i18n("Certificate &Groups"),
                                       i18nc("Only available in German and English. Leave to English for other languages.", "groupfeature_gnupgvsd_en.pdf"),
                                       QStringLiteral("../share/doc/gnupg-vsd"),
                                       QUrl(),
@@ -758,7 +761,7 @@ void MainWindow::Private::setupActions()
 #ifdef Q_OS_WIN
     const auto gpgol =
         new DocAction(QIcon::fromTheme(QStringLiteral("help-contextual")),
-                      i18n("&Mail encryption in Outlook"),
+                      i18n("&Mail Encryption in Outlook"),
                       i18nc("Only available in German and English. Leave to English for other languages. Only shown on Windows.", "gpgol_outlook_addin_en.pdf"),
                       QStringLiteral("../share/doc/gnupg-vsd"),
                       QUrl(),
@@ -778,7 +781,7 @@ void MainWindow::Private::setupActions()
 
     const auto smartcard =
         new DocAction(QIcon::fromTheme(QStringLiteral("help-contextual")),
-                      i18n("&Smartcard setup"),
+                      i18n("&Smartcard Setup"),
                       i18nc("Only available in German and English. Leave to English for other languages.", "smartcard_setup_gnupgvsd_en.pdf"),
                       QStringLiteral("../share/doc/gnupg-vsd"),
                       QUrl(),
@@ -796,30 +799,21 @@ void MainWindow::Private::setupActions()
     /* The secops */
     const auto approvalmanual =
         new DocAction(QIcon::fromTheme(QStringLiteral("dvipdf")),
-                      i18n("Manual for VS-NfD approval (German)"),
-                      i18nc("Only available in German. Keep German file name for all languages", "Handbuch-Zulassung-gnupgvsd-v3.2.pdf"),
+                      i18n("Manual for VS-NfD Approval"),
+                      i18nc("Only available in German and English. Keep the English file name for other languages.", "handbuch_zulassung_gnupgvsd_en.pdf"),
                       QStringLiteral("../share/doc/gnupg-vsd"),
                       QUrl(),
                       coll);
     coll->addAction(QStringLiteral("help_doc_approval_manual"), approvalmanual);
 
-    const auto vsa10573 =
+    const auto vsa =
         new DocAction(QIcon::fromTheme(QStringLiteral("dvipdf")),
-                      i18n("SecOps VSA-10573"),
-                      i18nc("Only available in German and English. Leave to English for other languages.", "BSI-VSA-10573-ENG_secops-20220207.pdf"),
+                      i18n("SecOps for VS-NfD Approval"),
+                      i18nc("Only available in German and English. Keep the English file name for other languages.", "BSI-VSA-10867_ENG_secops.pdf"),
                       QStringLiteral("../share/doc/gnupg-vsd"),
                       QUrl(),
                       coll);
-    coll->addAction(QStringLiteral("help_doc_vsa10573"), vsa10573);
-
-    const auto vsa10584 =
-        new DocAction(QIcon::fromTheme(QStringLiteral("dvipdf")),
-                      i18n("SecOps VSA-10584"),
-                      i18nc("Only available in German and English. Leave to English for other languages.", "BSI-VSA-10584-ENG_secops-20220207.pdf"),
-                      QStringLiteral("../share/doc/gnupg-vsd"),
-                      QUrl(),
-                      coll);
-    coll->addAction(QStringLiteral("help_doc_vsa10584"), vsa10584);
+    coll->addAction(QStringLiteral("help_doc_vsa"), vsa);
 
     q->setStandardToolBarMenuEnabled(true);
 
@@ -938,6 +932,11 @@ void MainWindow::dropEvent(QDropEvent *e)
 {
     qCDebug(KLEOPATRA_LOG);
 
+    if (e->source()) {
+        // The event comes from kleopatra itself; we don't want to import in this case.
+        return;
+    }
+
     if (!can_decode_local_files(e->mimeData())) {
         return;
     }
