@@ -10,6 +10,7 @@
 
 #include "p15cardwidget.h"
 
+#include "cardkeysview.h"
 #include "openpgpkeycardwidget.h"
 
 #include "settings.h"
@@ -18,12 +19,8 @@
 #include "smartcard/p15card.h"
 #include "smartcard/readerstatus.h"
 
-#include "commands/learncardkeyscommand.h"
-
-#include <QGridLayout>
 #include <QLabel>
 #include <QPushButton>
-#include <QScrollArea>
 #include <QStringList>
 #include <QVBoxLayout>
 
@@ -39,6 +36,7 @@
 #include <QGpgME/ImportFromKeyserverJob>
 #include <QGpgME/KeyListJob>
 #include <QGpgME/Protocol>
+
 #include <gpgme++/importresult.h>
 #include <gpgme++/keylistresult.h>
 
@@ -46,62 +44,34 @@
 
 using namespace Kleo;
 using namespace Kleo::SmartCard;
-using namespace Kleo::Commands;
 
 P15CardWidget::P15CardWidget(QWidget *parent)
-    : QWidget{parent}
-    , mVersionLabel{new QLabel{this}}
-    , mSerialNumber{new QLabel{this}}
-    , mStatusLabel{new QLabel{this}}
-    , mOpenPGPKeysSection{new QWidget{this}}
-    , mOpenPGPKeysWidget{new OpenPGPKeyCardWidget{this}}
+    : SmartCardWidget{parent}
 {
-    // Set up the scroll area
-    auto myLayout = new QVBoxLayout(this);
-    myLayout->setContentsMargins(0, 0, 0, 0);
-
-    auto area = new QScrollArea;
-    area->setFrameShape(QFrame::NoFrame);
-    area->setWidgetResizable(true);
-    myLayout->addWidget(area);
-
-    auto areaWidget = new QWidget;
-    area->setWidget(areaWidget);
-
-    auto areaVLay = new QVBoxLayout(areaWidget);
-
-    auto cardInfoGrid = new QGridLayout;
-    {
-        int row = 0;
-
-        // Version and Serialnumber
-        cardInfoGrid->addWidget(mVersionLabel, row++, 0, 1, 2);
-        mVersionLabel->setTextInteractionFlags(Qt::TextBrowserInteraction | Qt::TextSelectableByKeyboard);
-
-        cardInfoGrid->addWidget(new QLabel(i18n("Serial number:")), row, 0);
-        cardInfoGrid->addWidget(mSerialNumber, row++, 1);
-        mSerialNumber->setTextInteractionFlags(Qt::TextBrowserInteraction | Qt::TextSelectableByKeyboard);
-
-        cardInfoGrid->setColumnStretch(cardInfoGrid->columnCount(), 1);
-    }
-    areaVLay->addLayout(cardInfoGrid);
-    areaVLay->addWidget(mStatusLabel);
+    mStatusLabel = new QLabel{this};
     mStatusLabel->setVisible(false);
+    mContentLayout->addWidget(mStatusLabel);
 
-    areaVLay->addWidget(new KSeparator(Qt::Horizontal));
+    mContentLayout->addWidget(new KSeparator(Qt::Horizontal));
 
+    mOpenPGPKeysSection = new QWidget{this};
     {
         auto l = new QVBoxLayout{mOpenPGPKeysSection};
         l->setContentsMargins(0, 0, 0, 0);
         l->addWidget(new QLabel(QStringLiteral("<b>%1</b>").arg(i18n("OpenPGP keys:"))));
+        mOpenPGPKeysWidget = new OpenPGPKeyCardWidget{this};
         mOpenPGPKeysWidget->setAllowedActions(OpenPGPKeyCardWidget::NoAction);
         l->addWidget(mOpenPGPKeysWidget);
         l->addWidget(new KSeparator(Qt::Horizontal));
     }
     mOpenPGPKeysSection->setVisible(false);
-    areaVLay->addWidget(mOpenPGPKeysSection);
+    mContentLayout->addWidget(mOpenPGPKeysSection);
 
-    areaVLay->addStretch(1);
+    mCardKeysView = new CardKeysView{this};
+    mCardKeysView->setVisible(false);
+    mContentLayout->addWidget(mCardKeysView);
+
+    mContentLayout->addStretch(1);
 }
 
 P15CardWidget::~P15CardWidget() = default;
@@ -111,7 +81,7 @@ void P15CardWidget::searchPGPFpr(const std::string &fpr)
     /* Only do auto import from LDAP */
     auto conf = QGpgME::cryptoConfig();
     Q_ASSERT(conf);
-    if (!Settings().alwaysSearchCardOnKeyserver() && !Kleo::keyserver().startsWith(QLatin1String{"ldap"})) {
+    if (!Settings().alwaysSearchCardOnKeyserver() && !Kleo::keyserver().startsWith(QLatin1StringView{"ldap"})) {
         return;
     }
     mStatusLabel->setText(i18n("Searching in directory service..."));
@@ -145,10 +115,7 @@ void P15CardWidget::searchPGPFpr(const std::string &fpr)
 
 void P15CardWidget::setCard(const P15Card *card)
 {
-    mCardSerialNumber = card->serialNumber();
-    mVersionLabel->setText(i18nc("%1 is a smartcard manufacturer", "%1 PKCS#15 card", QString::fromStdString(card->manufacturer())));
-    mSerialNumber->setText(card->displaySerialNumber());
-    mSerialNumber->setToolTip(QString::fromStdString(card->serialNumber()));
+    SmartCardWidget::setCard(card);
 
     const auto sigInfo = card->keyInfo(card->signingKeyRef());
     if (!sigInfo.grip.empty()) {
@@ -176,21 +143,8 @@ void P15CardWidget::setCard(const P15Card *card)
     if (!Settings().autoLoadP15Certs()) {
         return;
     }
-    for (const auto &info : card->keyInfos()) {
-        const auto key = KeyCache::instance()->findSubkeyByKeyGrip(info.grip);
-        if (key.isNull()) {
-            auto cmd = new LearnCardKeysCommand(GpgME::CMS);
-            cmd->setParentWidget(this);
-            cmd->setShowsOutputWindow(false);
-            qCDebug(KLEOPATRA_LOG) << "Did not find:" << info.grip.c_str() << "Starting gpgsm --learn.";
-            cmd->start();
-            connect(cmd, &Command::finished, this, []() {
-                qCDebug(KLEOPATRA_LOG) << "Learn command finished.";
-            });
-            return;
-        }
-    }
-    qCDebug(KLEOPATRA_LOG) << "All certificates from card cached - Not learning.";
+    mCardKeysView->setVisible(true);
+    mCardKeysView->setCard(card);
 }
 
 #include "moc_p15cardwidget.cpp"
