@@ -18,7 +18,6 @@
 #include "kleopatra_debug.h"
 
 #include "commands/createcsrforcardkeycommand.h"
-#include "commands/createopenpgpkeyfromcardkeyscommand.h"
 #include "commands/openpgpgeneratecardkeycommand.h"
 
 #include "smartcard/algorithminfo.h"
@@ -39,7 +38,6 @@
 #include <QLabel>
 #include <QProgressDialog>
 #include <QPushButton>
-#include <QScrollArea>
 #include <QThread>
 #include <QVBoxLayout>
 
@@ -140,158 +138,145 @@ private:
 } // Namespace
 
 PGPCardWidget::PGPCardWidget(QWidget *parent)
-    : QWidget(parent)
-    , mSerialNumber(new QLabel(this))
-    , mCardHolderLabel(new QLabel(this))
-    , mVersionLabel(new QLabel(this))
-    , mUrlLabel(new QLabel(this))
-    , mCardIsEmpty(false)
+    : SmartCardWidget(parent)
 {
-    // Set up the scroll area
-    auto myLayout = new QVBoxLayout(this);
-    myLayout->setContentsMargins(0, 0, 0, 0);
-
-    auto area = new QScrollArea;
-    area->setFrameShape(QFrame::NoFrame);
-    area->setWidgetResizable(true);
-    myLayout->addWidget(area);
-
-    auto areaWidget = new QWidget;
-    area->setWidget(areaWidget);
-
-    auto areaVLay = new QVBoxLayout(areaWidget);
-
-    auto cardInfoGrid = new QGridLayout;
     {
-        int row = 0;
-
-        // Version and Serialnumber
-        cardInfoGrid->addWidget(mVersionLabel, row, 0, 1, 2);
-        mVersionLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
-        row++;
-
-        cardInfoGrid->addWidget(new QLabel(i18n("Serial number:")), row, 0);
-        cardInfoGrid->addWidget(mSerialNumber, row, 1);
-        mSerialNumber->setTextInteractionFlags(Qt::TextBrowserInteraction);
-        row++;
+        mInfoGridLayout->setColumnStretch(mInfoGridLayout->columnCount() - 1, 0); // undo stretch set by base widget
+        int row = mInfoGridLayout->rowCount();
 
         // Cardholder Row
-        cardInfoGrid->addWidget(new QLabel(i18nc("The owner of a smartcard. GnuPG refers to this as cardholder.", "Cardholder:")), row, 0);
-        cardInfoGrid->addWidget(mCardHolderLabel, row, 1);
+        mInfoGridLayout->addWidget(new QLabel(i18nc("The owner of a smartcard. GnuPG refers to this as cardholder.", "Cardholder:")), row, 0);
+        mCardHolderLabel = new QLabel{this};
         mCardHolderLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
+        mInfoGridLayout->addWidget(mCardHolderLabel, row, 1);
         {
-            auto button = new QPushButton;
+            auto button = new QPushButton{this};
             button->setIcon(QIcon::fromTheme(QStringLiteral("cell_edit")));
             button->setAccessibleName(i18nc("@action:button", "Edit"));
-            button->setToolTip(i18n("Change"));
-            cardInfoGrid->addWidget(button, row, 2);
+            button->setToolTip(i18nc("@info:tooltip", "Change"));
+            mInfoGridLayout->addWidget(button, row, 2);
             connect(button, &QPushButton::clicked, this, &PGPCardWidget::changeNameRequested);
         }
         row++;
 
         // URL Row
-        cardInfoGrid->addWidget(new QLabel(i18nc("The URL under which a public key that "
-                                                 "corresponds to a smartcard can be downloaded",
-                                                 "Pubkey URL:")),
-                                row,
-                                0);
-        cardInfoGrid->addWidget(mUrlLabel, row, 1);
+        mInfoGridLayout->addWidget(new QLabel(i18nc("The URL under which a public key that "
+                                                    "corresponds to a smartcard can be downloaded",
+                                                    "Pubkey URL:")),
+                                   row,
+                                   0);
+        mUrlLabel = new QLabel{this};
         mUrlLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
+        mInfoGridLayout->addWidget(mUrlLabel, row, 1);
         {
-            auto button = new QPushButton;
+            auto button = new QPushButton{this};
             button->setIcon(QIcon::fromTheme(QStringLiteral("cell_edit")));
             button->setAccessibleName(i18nc("@action:button", "Edit"));
-            button->setToolTip(i18n("Change"));
-            cardInfoGrid->addWidget(button, row, 2);
+            button->setToolTip(i18nc("@info:tooltip", "Change"));
+            mInfoGridLayout->addWidget(button, row, 2);
             connect(button, &QPushButton::clicked, this, &PGPCardWidget::changeUrlRequested);
         }
+        row++;
 
-        cardInfoGrid->setColumnStretch(cardInfoGrid->columnCount(), 1);
+        // PIN counters row
+        {
+            mInfoGridLayout->addWidget(new QLabel(i18nc("@label The number of remaining attempts to enter a PIN or PUK, as in "
+                                                        "Remaining attempts: PIN: 2, PUK: 3, Admin PIN: 3",
+                                                        "Remaining attempts:")),
+                                       row,
+                                       0);
+            mPinCounterLabel = new QLabel{this};
+            mPinCounterLabel->setToolTip(xi18nc("@info:tooltip", "Shows the number of remaining attempts for entering the correct PIN or PUK."));
+            mPinCounterLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
+            mInfoGridLayout->addWidget(mPinCounterLabel, row, 1);
+        }
+
+        mInfoGridLayout->setColumnStretch(mInfoGridLayout->columnCount(), 1);
     }
-    areaVLay->addLayout(cardInfoGrid);
 
-    areaVLay->addWidget(new KSeparator(Qt::Horizontal));
+    mContentLayout->addWidget(new KSeparator(Qt::Horizontal));
 
     // The keys
-    areaVLay->addWidget(new QLabel(QStringLiteral("<b>%1</b>").arg(i18n("Keys:"))));
+    mContentLayout->addWidget(new QLabel(QStringLiteral("<b>%1</b>").arg(i18n("Keys:"))));
 
     mKeysWidget = new OpenPGPKeyCardWidget{this};
-    areaVLay->addWidget(mKeysWidget);
+    mContentLayout->addWidget(mKeysWidget);
     connect(mKeysWidget, &OpenPGPKeyCardWidget::createCSRRequested, this, &PGPCardWidget::createCSR);
     connect(mKeysWidget, &OpenPGPKeyCardWidget::generateKeyRequested, this, &PGPCardWidget::generateKey);
 
-    areaVLay->addWidget(new KSeparator(Qt::Horizontal));
+    mContentLayout->addWidget(new KSeparator(Qt::Horizontal));
 
-    areaVLay->addWidget(new QLabel(QStringLiteral("<b>%1</b>").arg(i18n("Actions:"))));
+    mContentLayout->addWidget(new QLabel(QStringLiteral("<b>%1</b>").arg(i18n("Actions:"))));
 
     auto actionLayout = new QHBoxLayout;
 
     {
-        auto generateButton = new QPushButton(i18n("Generate New Keys"));
-        generateButton->setToolTip(i18n("Create a new primary key and generate subkeys on the card."));
+        auto generateButton = new QPushButton(i18nc("@action:button", "Generate New Keys"), this);
+        generateButton->setToolTip(xi18nc("@info:tooltip",
+                                          "<para>Generate three new keys on the smart card and create a new OpenPGP "
+                                          "certificate with those keys. Optionally, the encryption key is generated "
+                                          "off-card and a backup is created so that you can still access data encrypted "
+                                          "with this key in case the card is lost or damaged.</para>"
+                                          "<para><emphasis strong='true'>"
+                                          "Existing keys on the smart card will be overwritten."
+                                          "</emphasis></para>"));
         actionLayout->addWidget(generateButton);
         connect(generateButton, &QPushButton::clicked, this, &PGPCardWidget::genkeyRequested);
     }
     {
-        auto pinButton = new QPushButton(i18n("Change PIN"));
-        pinButton->setToolTip(i18n("Change the PIN required for using the keys on the smartcard."));
+        auto pinButton = new QPushButton(i18nc("@action:button", "Change PIN"), this);
+        pinButton->setToolTip(i18nc("@info:tooltip",
+                                    "Change the PIN required for using the keys on the smart card. "
+                                    "The PIN must contain at least six characters."));
         actionLayout->addWidget(pinButton);
         connect(pinButton, &QPushButton::clicked, this, [this]() {
             doChangePin(OpenPGPCard::pinKeyRef());
         });
     }
     {
-        auto unblockButton = new QPushButton(i18n("Unblock Card"));
-        unblockButton->setToolTip(i18n("Unblock the smartcard and set a new PIN."));
+        auto unblockButton = new QPushButton(i18nc("@action:button", "Unblock Card"), this);
+        unblockButton->setToolTip(i18nc("@info:tooltip", "Unblock the smart card with the PUK (if available) or the Admin PIN."));
         actionLayout->addWidget(unblockButton);
         connect(unblockButton, &QPushButton::clicked, this, [this]() {
-            doChangePin(OpenPGPCard::resetCodeKeyRef());
+            if (mPUKIsAvailable) {
+                // unblock card with the PUK
+                doChangePin(OpenPGPCard::resetCodeKeyRef());
+            } else {
+                // unblock card with the Admin PIN
+                doChangePin(OpenPGPCard::pinKeyRef(), ChangePinCommand::ResetMode);
+            }
         });
     }
     {
-        auto pukButton = new QPushButton(i18n("Change Admin PIN"));
-        pukButton->setToolTip(i18n("Change the PIN required for administrative operations."));
+        auto pukButton = new QPushButton(i18nc("@action:button", "Change Admin PIN"), this);
+        pukButton->setToolTip(i18nc("@info:tooltip", "Change the PIN required for administrative operations."));
         actionLayout->addWidget(pukButton);
         connect(pukButton, &QPushButton::clicked, this, [this]() {
             doChangePin(OpenPGPCard::adminPinKeyRef());
         });
     }
     {
-        auto resetCodeButton = new QPushButton(i18n("Change Reset Code"));
-        resetCodeButton->setToolTip(i18n("Change the PIN required to unblock the smartcard and set a new PIN."));
-        actionLayout->addWidget(resetCodeButton);
-        connect(resetCodeButton, &QPushButton::clicked, this, [this]() {
+        mSetOrChangePUKButton = new QPushButton(i18nc("@action:button", "Set PUK"), this);
+        mSetOrChangePUKButton->setToolTip(i18nc("@info:tooltip",
+                                                "Set or change the PUK that can be used to unblock the smart card. "
+                                                "The PUK must contain at least eight characters."));
+        actionLayout->addWidget(mSetOrChangePUKButton);
+        connect(mSetOrChangePUKButton, &QPushButton::clicked, this, [this]() {
             doChangePin(OpenPGPCard::resetCodeKeyRef(), ChangePinCommand::ResetMode);
         });
     }
 
-    if (CreateOpenPGPKeyFromCardKeysCommand::isSupported()) {
-        mKeyForCardKeysButton = new QPushButton(this);
-        mKeyForCardKeysButton->setText(i18n("Create OpenPGP Key"));
-        mKeyForCardKeysButton->setToolTip(i18n("Create an OpenPGP key for the keys stored on the card."));
-        actionLayout->addWidget(mKeyForCardKeysButton);
-        connect(mKeyForCardKeysButton, &QPushButton::clicked, this, &PGPCardWidget::createKeyFromCardKeys);
-    }
-
     actionLayout->addStretch(-1);
-    areaVLay->addLayout(actionLayout);
+    mContentLayout->addLayout(actionLayout);
 
-    areaVLay->addStretch(1);
+    mContentLayout->addStretch(1);
 }
 
 void PGPCardWidget::setCard(const OpenPGPCard *card)
 {
-    const QString version = card->displayAppVersion();
+    SmartCardWidget::setCard(card);
 
     mIs21 = card->appVersion() >= 0x0201;
-    const QString manufacturer = QString::fromStdString(card->manufacturer());
-    const bool manufacturerIsUnknown = manufacturer.isEmpty() || manufacturer == QLatin1StringView("unknown");
-    mVersionLabel->setText(
-        manufacturerIsUnknown
-            ? i18nc("Placeholder is a version number", "Unknown OpenPGP v%1 card", version)
-            : i18nc("First placeholder is manufacturer, second placeholder is a version number", "%1 OpenPGP v%2 card", manufacturer, version));
-    mSerialNumber->setText(card->displaySerialNumber());
-    mRealSerial = card->serialNumber();
 
     const auto holder = card->cardHolder();
     const auto url = QString::fromStdString(card->pubkeyUrl());
@@ -300,22 +285,30 @@ void PGPCardWidget::setCard(const OpenPGPCard *card)
     mUrlLabel->setText(url.isEmpty() ? i18n("not set") : QStringLiteral("<a href=\"%1\">%1</a>").arg(url.toHtmlEscaped()));
     mUrlLabel->setOpenExternalLinks(true);
 
+    const auto pinLabels = card->pinLabels();
+    const auto pinCounters = card->pinCounters();
+    QStringList countersWithLabels;
+    countersWithLabels.reserve(pinCounters.size());
+    for (const auto &pinCounter : pinCounters) {
+        // sanity check
+        if (countersWithLabels.size() == pinLabels.size()) {
+            break;
+        }
+        countersWithLabels.push_back(i18nc("label: value", "%1: %2", pinLabels[countersWithLabels.size()], pinCounter));
+    }
+    mPinCounterLabel->setText(countersWithLabels.join(QLatin1String(", ")));
+    mPUKIsAvailable = (pinCounters.size() == 3) && (pinCounters[1] > 0);
+    mSetOrChangePUKButton->setText(mPUKIsAvailable ? i18nc("@action:button", "Change PUK") : i18nc("@action:button", "Set PUK"));
+
     mKeysWidget->update(card);
 
     mCardIsEmpty = card->keyFingerprint(OpenPGPCard::pgpSigKeyRef()).empty() && card->keyFingerprint(OpenPGPCard::pgpEncKeyRef()).empty()
         && card->keyFingerprint(OpenPGPCard::pgpAuthKeyRef()).empty();
-
-    if (mKeyForCardKeysButton) {
-        mKeyForCardKeysButton->setEnabled(card->hasSigningKey() //
-                                          && card->hasEncryptionKey() //
-                                          && DeVSCompliance::algorithmIsCompliant(card->keyInfo(card->signingKeyRef()).algorithm)
-                                          && DeVSCompliance::algorithmIsCompliant(card->keyInfo(card->encryptionKeyRef()).algorithm));
-    }
 }
 
 void PGPCardWidget::doChangePin(const std::string &keyRef, ChangePinCommand::ChangePinMode mode)
 {
-    auto cmd = new ChangePinCommand(mRealSerial, OpenPGPCard::AppName, this);
+    auto cmd = new ChangePinCommand(mSerialNumber, OpenPGPCard::AppName, this);
     this->setEnabled(false);
     connect(cmd, &ChangePinCommand::finished, this, [this]() {
         this->setEnabled(true);
@@ -327,7 +320,7 @@ void PGPCardWidget::doChangePin(const std::string &keyRef, ChangePinCommand::Cha
 
 void PGPCardWidget::doGenKey(GenCardKeyDialog *dlg)
 {
-    const GpgME::Error err = ReaderStatus::switchCardAndApp(mRealSerial, OpenPGPCard::AppName);
+    const GpgME::Error err = ReaderStatus::switchCardAndApp(mSerialNumber, OpenPGPCard::AppName);
     if (err) {
         return;
     }
@@ -342,8 +335,8 @@ void PGPCardWidget::doGenKey(GenCardKeyDialog *dlg)
     progress->setModal(true);
     progress->setCancelButton(nullptr);
     progress->setWindowTitle(i18nc("@title:window", "Generating Keys"));
-    progress->setLabel(new QLabel(i18n("This may take several minutes...")));
-    auto workerThread = new GenKeyThread(params, mRealSerial);
+    progress->setLabel(new QLabel(i18nc("@label:textbox", "This may take several minutes...")));
+    auto workerThread = new GenKeyThread(params, mSerialNumber);
     connect(workerThread, &QThread::finished, this, [this, workerThread, progress] {
         progress->accept();
         progress->deleteLater();
@@ -381,9 +374,9 @@ void PGPCardWidget::genKeyDone(const GpgME::Error &err, const std::string &backu
 
 void PGPCardWidget::genkeyRequested()
 {
-    const auto pgpCard = ReaderStatus::instance()->getCard<OpenPGPCard>(mRealSerial);
+    const auto pgpCard = ReaderStatus::instance()->getCard<OpenPGPCard>(mSerialNumber);
     if (!pgpCard) {
-        KMessageBox::error(this, i18n("Failed to find the OpenPGP card with the serial number: %1", QString::fromStdString(mRealSerial)));
+        KMessageBox::error(this, i18n("Failed to find the OpenPGP card with the serial number: %1", QString::fromStdString(mSerialNumber)));
         return;
     }
 
@@ -447,9 +440,9 @@ void PGPCardWidget::changeNameRequested()
     const auto lastName = parts.takeLast();
     const QString formatted = lastName + QStringLiteral("<<") + parts.join(QLatin1Char('<'));
 
-    const auto pgpCard = ReaderStatus::instance()->getCard<OpenPGPCard>(mRealSerial);
+    const auto pgpCard = ReaderStatus::instance()->getCard<OpenPGPCard>(mSerialNumber);
     if (!pgpCard) {
-        KMessageBox::error(this, i18n("Failed to find the OpenPGP card with the serial number: %1", QString::fromStdString(mRealSerial)));
+        KMessageBox::error(this, i18n("Failed to find the OpenPGP card with the serial number: %1", QString::fromStdString(mSerialNumber)));
         return;
     }
 
@@ -494,9 +487,9 @@ void PGPCardWidget::changeUrlRequested()
         break;
     }
 
-    const auto pgpCard = ReaderStatus::instance()->getCard<OpenPGPCard>(mRealSerial);
+    const auto pgpCard = ReaderStatus::instance()->getCard<OpenPGPCard>(mSerialNumber);
     if (!pgpCard) {
-        KMessageBox::error(this, i18n("Failed to find the OpenPGP card with the serial number: %1", QString::fromStdString(mRealSerial)));
+        KMessageBox::error(this, i18n("Failed to find the OpenPGP card with the serial number: %1", QString::fromStdString(mSerialNumber)));
         return;
     }
 
@@ -518,19 +511,9 @@ void PGPCardWidget::changeUrlResult(const GpgME::Error &err)
     }
 }
 
-void PGPCardWidget::createKeyFromCardKeys()
-{
-    auto cmd = new CreateOpenPGPKeyFromCardKeysCommand(mRealSerial, OpenPGPCard::AppName, this);
-    this->setEnabled(false);
-    connect(cmd, &CreateOpenPGPKeyFromCardKeysCommand::finished, this, [this]() {
-        this->setEnabled(true);
-    });
-    cmd->start();
-}
-
 void PGPCardWidget::createCSR(const std::string &keyref)
 {
-    auto cmd = new CreateCSRForCardKeyCommand(keyref, mRealSerial, OpenPGPCard::AppName, this);
+    auto cmd = new CreateCSRForCardKeyCommand(keyref, mSerialNumber, OpenPGPCard::AppName, this);
     this->setEnabled(false);
     connect(cmd, &CreateCSRForCardKeyCommand::finished, this, [this]() {
         this->setEnabled(true);
@@ -540,7 +523,7 @@ void PGPCardWidget::createCSR(const std::string &keyref)
 
 void PGPCardWidget::generateKey(const std::string &keyref)
 {
-    auto cmd = new OpenPGPGenerateCardKeyCommand(keyref, mRealSerial, this);
+    auto cmd = new OpenPGPGenerateCardKeyCommand(keyref, mSerialNumber, this);
     this->setEnabled(false);
     connect(cmd, &OpenPGPGenerateCardKeyCommand::finished, this, [this]() {
         this->setEnabled(true);
